/**
 * @ingroup   sc68app_devel
 * @file      SC68delay.c
 * @author    Ben(jamin) Gerard <ben@sashipa.com>
 * @date      1999/07/27
 * @brief     Delay processing functions
 * @version   $Id: SC68delay.c,v 1.1.1.1 2001/12/24 20:50:11 ben Exp $
 */

/*
 *                    sc68 - Delay processing functions
 *         Copyright (C) 2001 Ben(jamin) Gerard <ben@sashipa.com>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


#include <math.h>

#include "sc68app/SC68delay.h"
#include "file68/SC68os_general.h"
#include "file68/SC68error.h"
#include "file68/SC68utils.h"

#define DELAY_DEFAULT_BUFFER_SIZE  0x4000
#define DELAY_DEFAULT_STRENGHT     164
#define DELAY_MAX_BUFFER           delaybufsz
#define DELAY_BUFFER_MASK          delaybufmsk

static volatile int init = 0;
static int *delaybuf, delayptr;
static int delaybufsz,delaybufmsk;

/* Small Bijection for calculate a square root */
static unsigned int Sqrt(unsigned int x)
{
  unsigned int r = 0;
  unsigned int t;

  for (t=0x40000000; t; t>>= 2) {
    unsigned int s = r + t;
    if (s <= x) {
      x -= s;
      r = s + t;
    }
    r >>= 1;
  }
  return r;
}


/* Create delay buffer, size is round to a greater or equal power of 2
 * return new buffer size
 */
int SC68delay_init(int buffer_size)
{
  int i;
  if(init)
    return 0;
  for(i=0x800000; i && !(i&buffer_size); i>>=1);
  if(!i) delaybufsz = DELAY_DEFAULT_BUFFER_SIZE;
  else   delaybufsz = i<<(buffer_size!=i);
  delaybufmsk = delaybufsz-1;
  SC68os_pdebug("Creating delay buffers : %d samples / buffer", delaybufsz);
  delayptr = 0;
  delaybuf = (int *)SC68_malloc(delaybufsz * sizeof(*delaybuf));
  if(delaybuf==NULL)
    return SC68error_add("SC68delay_init(%d) : Memory failure", delaybufsz);
  init = 1;
  return delaybufsz;
}

/* Kill delay buffers
 */
void SC68delay_kill()
{
  if(!init)
    return;
  delayptr = 0;
  SC68_free(delaybuf);
  delaybuf=NULL;
  delaybufsz = 0;
  delaybufmsk = 0;
  init = 0;
}

/* Apply a Left to right delay
 * buffer : L-R buffer to delay
 * nb     : nb sample in buffer
 * delay  : delay value ( in sample )
 */
void SC68delay_LR(u32 *buffer, int nb, int delay)
{
  if(!init) return;
  if(nb<=0 || delay==0) return;
  if((unsigned)delay>(unsigned)(DELAY_MAX_BUFFER-nb)) delay = DELAY_MAX_BUFFER-nb;
  do
  {
    u32 v,rd;

    v = *buffer;
    rd = delaybuf[(delayptr-delay)&DELAY_BUFFER_MASK] & 0xFFFF;
    delaybuf[delayptr] = v;
    *buffer++ = (v&0xFFFF0000) + rd;
    delayptr = (delayptr+1)&DELAY_BUFFER_MASK;
  } while(--nb);
}

/* Apply delay to buffer
 * buffer   : L-R buffer to delay
 * nb       : nb sample in buffer
 * delay    : delay value ( in sample )
 * strength : mix buffer volume [0..256]
 */
void SC68delay(u32 *buffer, int nb, int delay, int strength, int sign)
{
  int scale;

  if(!init) return;
  if(nb<=0 || delay==0 || strength==0) return;
  if((unsigned)strength>256) strength = DELAY_DEFAULT_STRENGHT;
  if((unsigned)delay>(unsigned)(DELAY_MAX_BUFFER-nb)) delay = DELAY_MAX_BUFFER-nb;

  scale = Sqrt( (256+strength) << 8 );
  scale >>= 2;  /* 0 .. 128 */

  do
  {
    u32 l,r,rd,ld;
    int extend;

    l   = *buffer ^ sign;       /* unsigned value */
    r   = (u16)l;
    l >>= 16;
    l <<= 6;                    /* FIX 6 */
    r <<= 6;

    ld = delaybuf[(delayptr-delay)&DELAY_BUFFER_MASK];
    rd = ((u16)ld  * strength) >> 2; /* FIX 6 */
    ld = ((ld>>16) * strength) >> 2;

    rd += r;
    ld += l;

    rd = ((rd + r) * scale) >> 13;
    ld = ((ld + l) * scale) >> 13;

    extend = (int)(0xFFFF - rd) >> ((sizeof(int)<<3)-1);
    rd |= extend;

    extend = (int)(0xFFFF - ld) >> ((sizeof(int)<<3)-1);
    ld |= extend;

    ld = ((ld<<16) | (rd&0xFFFF)); /* unsigned result in delay buffer */
    delaybuf[delayptr] = ld;       /* sign (or not!) for output */
    *buffer++ = ld ^ sign;

    delayptr = (delayptr+1)&DELAY_BUFFER_MASK;
  } while(--nb);
}

/*  Get Delay ring default strength
 */
int SC68delay_default_strength(void)
{
  return DELAY_DEFAULT_STRENGHT;
}

/*  Get Delay ring default strength
 */
int SC68delay_default_buffer_size(void)
{
  return DELAY_DEFAULT_BUFFER_SIZE;
}
