/**
 * @ingroup   sc68app_devel
 * @file      SC68mixer.c
 * @author    Ben(jamin) Gerard <ben@sashipa.com>
 * @date      1999/05/17
 * @brief     sc68 - audio mixer
 * @version   $Id: SC68mixer.c,v 1.1.1.1 2001/12/24 20:50:11 ben Exp $
 */

/*
 *                        sc68 - sc68 - audio mixer
 *         Copyright (C) 2001 Ben(jamin) Gerard <ben@sashipa.com>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "sc68app/SC68mixer.h"

/* V MUST BE UNSIGNED 32 BIT */

/* ARM compliant version */
/* #define SWAP_16BITWORD(V) ((V^=V<<16), (V^=V>>16), (V^=V<<16)) */

#define SWAP_16BITWORD(V) ((V>>16)+(V<<16))

/*  Mix 16-bit-stereo value into 16-bit-stereo
 *  res = buf^sign
 *  sign=0          : Keep input sign, 
 *  sign=0x00008000 : Change left channel sign
 *  sign=0x80000000 : Change right channel sign
 *  sign=0x80008000 : Change both channel
 */
void SC68mixer_stereo_16_LR(u32 *dest, u32 *src, int nb, u32 sign)
{
  u32 *end;
  int rem;

  rem = nb&3;
  end = dest+(nb-rem);
  if(dest<end)
    do
    {
      *dest++ = (*src++)^sign;
      *dest++ = (*src++)^sign;
      *dest++ = (*src++)^sign;
      *dest++ = (*src++)^sign;
    } while(dest<end);

    if(rem&1)
      *dest++ = (*src++)^sign;
    if(rem&2)
    {
      *dest++ = (*src++)^sign;
      *dest++ = (*src++)^sign;
    }
}

/*  Mix 32-bit-stereo value into 16-bit-stereo with CHANNEL SWAPPING
 */
void SC68mixer_stereo_16_RL(u32 *dest, u32 *src, int nb, u32 sign)
{
  u32 *end;
  int rem;

  if(dest==src && !sign)
    return;
  rem = nb&3;
  end = dest+(nb-rem);
  if(dest<end)
    do
    {
      u32 v;
      v = (*src++)^sign;
      *dest++ = SWAP_16BITWORD(v);
      v = (*src++)^sign;
      *dest++ = SWAP_16BITWORD(v);
      v = (*src++)^sign;
      *dest++ = SWAP_16BITWORD(v);
      v = (*src++)^sign;
      *dest++ = SWAP_16BITWORD(v);
    } while(dest<end);

    if(rem&1)
    {
      u32 v;
      v = (*src++)^sign;
      *dest++ = SWAP_16BITWORD(v);
    }
    if(rem&2)
    {
      u32 v;
      v = (*src++)^sign;
      *dest++ = SWAP_16BITWORD(v);
      v = (*src++)^sign;
      *dest++ = SWAP_16BITWORD(v);
    }
}

/*  Mix 16-bit-stereo value into 32-bit-stereo-float (-1..1)
 */
void SC68mixer_stereo_FL_LR(float *dest, u32 *src, int nb, u32 sign)
{
  const float mult = 1.0f/32768.0f;
  //  const float mult = 1.0f/65536.0f;
  int v;
  float *end;

  end = dest+(nb*2);
  if(dest<end)
    do
    {
      v = *src++ ^ sign;
      dest[0] = (float)(s16)(v) * mult;
      dest[1] = (float)(v>>16) * mult;
      dest += 2;
    } while(dest<end);
}

/*  Duplicate left channel into right channel before EOR transform
 *  res = (buf|(buf<<16))^sign
 */
void SC68mixer_dup_L_to_R(u32 *dest, u32 *src, int nb, u32 sign)
{
  u32 *end;
  int rem;

  rem = nb&3;
  end = dest+(nb-rem);
  if(dest<end)
    do
    {
      u32 v;
      v=(u16)*src++; *dest++ = ((v<<16)|v)^sign;
      v=(u16)*src++; *dest++ = ((v<<16)|v)^sign;
      v=(u16)*src++; *dest++ = ((v<<16)|v)^sign;
      v=(u16)*src++; *dest++ = ((v<<16)|v)^sign;
    } while(dest<end);

    if(rem&1)
    {
      u32 v;
      v=(u16)*src++; *dest++ = ((v<<16)|v)^sign;
    }
    if(rem&2)
    {
      u32 v;
      v=(u16)*src++; *dest++ = ((v<<16)|v)^sign;
      v=(u16)*src++; *dest++ = ((v<<16)|v)^sign;
    }
}

/*  Duplicate right channel into left channel before EOR transform
 *  res = (buf|(buf>>16))^sign
 */
void SC68mixer_dup_R_to_L(u32 *dest, u32 *src, int nb, u32 sign)
{
  u32 *end;
  int rem;

  rem = nb&3;
  end = dest+(nb-rem);
  if(dest<end)
    do
    {
      u32 v;
      v=*src++&0xFFFF0000; *dest++ = ((v>>16)|v)^sign;
      v=*src++&0xFFFF0000; *dest++ = ((v>>16)|v)^sign;
      v=*src++&0xFFFF0000; *dest++ = ((v>>16)|v)^sign;
      v=*src++&0xFFFF0000; *dest++ = ((v>>16)|v)^sign;
    } while(dest<end);

    if(rem&1)
    {
      u32 v;
      v=*src++&0xFFFF0000; *dest++ = ((v>>16)|v)^sign;
    }
    if(rem&2)
    {
      u32 v;
      v=*src++&0xFFFF0000; *dest++ = ((v>>16)|v)^sign;
      v=*src++&0xFFFF0000; *dest++ = ((v>>16)|v)^sign;
    }
}

/*  Blend Left and right voice :
 *  factor [0..65536], 0:blend nothing, 65536:swap L/R
 */
void SC68mixer_blend_LR(u32 *dest, u32 *src, int nb, int factor, u32 sign)
{
  u32 *end;
  int rem;
  int oof;

  if(factor<=0) {
    if(sign) {
      SC68mixer_stereo_16_LR(dest, src, nb, sign);
    }
    return;
  } else if(factor>=65536) {
    SC68mixer_stereo_16_RL(dest, src, nb, sign);
    return;
  }

  oof = 65536-factor;
  rem = nb&3;
  end = dest+(nb-rem);
  if(dest<end)
    do
    {
      u32 l,r,v;
      v=*src++; l=v>>16; r=(u16)v; *dest++ = (((l*oof+r*factor)&0xFFFF0000) | (((r*oof+l*factor))>>16))^sign;
      v=*src++; l=v>>16; r=(u16)v; *dest++ = (((l*oof+r*factor)&0xFFFF0000) | (((r*oof+l*factor))>>16))^sign;
      v=*src++; l=v>>16; r=(u16)v; *dest++ = (((l*oof+r*factor)&0xFFFF0000) | (((r*oof+l*factor))>>16))^sign;
      v=*src++; l=v>>16; r=(u16)v; *dest++ = (((l*oof+r*factor)&0xFFFF0000) | (((r*oof+l*factor))>>16))^sign;
    } while(dest<end);

    if(rem&1)
    {
      u32 l,r,v;
      v=*src++; l=v>>16; r=(u16)v; *dest++ = (((l*oof+r*factor)&0xFFFF0000) | (((r*oof+l*factor))>>16))^sign;
    }
    if(rem&2)
    {
      u32 l,r,v;
      v=*src++; l=v>>16; r=(u16)v; *dest++ = (((l*oof+r*factor)&0xFFFF0000) | (((r*oof+l*factor))>>16))^sign;
      v=*src++; l=v>>16; r=(u16)v; *dest++ = (((l*oof+r*factor)&0xFFFF0000) | (((r*oof+l*factor))>>16))^sign;
    }
}

/*  Multiply left/right channel by ml/mr factor [0..65536]
 */
void SC68mixer_mult_LR(u32 *dest, u32 *src, int nb, unsigned ml, unsigned mr, u32 sign)
{
  u32 *end;
  int rem;
  if(ml==65536 && mr==65536)
  {
    SC68mixer_stereo_16_LR(dest,src,nb,sign);
    return;
  }

  if(ml==0 && mr==0)
  {
    SC68mixer_fill(dest,nb,sign);
    return;
  }

  rem = nb&3;
  end = dest+(nb-rem);

  if(dest<end)
    do
    {
      u32 l,r,v;
      v=*src++; l=(u16)v; r=v>>16; *dest++ = (((l*ml)>>16) | ((r*mr)&0xFFFF0000))^sign;
      v=*src++; l=(u16)v; r=v>>16; *dest++ = (((l*ml)>>16) | ((r*mr)&0xFFFF0000))^sign;
      v=*src++; l=(u16)v; r=v>>16; *dest++ = (((l*ml)>>16) | ((r*mr)&0xFFFF0000))^sign;
      v=*src++; l=(u16)v; r=v>>16; *dest++ = (((l*ml)>>16) | ((r*mr)&0xFFFF0000))^sign;
    } while(dest<end);

    if(rem&1)
    {
      u32 l,r,v;
      v=*src++; l=(u16)v; r=v>>16; *dest++ = (((l*ml)>>16) | ((r*mr)&0xFFFF0000))^sign;
    }
    if(rem&2)
    {
      u32 l,r,v;
      v=*src++; l=(u16)v; r=v>>16; *dest++ = (((l*ml)>>16) | ((r*mr)&0xFFFF0000))^sign;
      v=*src++; l=(u16)v; r=v>>16; *dest++ = (((l*ml)>>16) | ((r*mr)&0xFFFF0000))^sign;
    }
}

/*  Fill buffer sign with value (RRRRLLLL)
 */
void SC68mixer_fill(u32 *dest, int nb, u32 sign)
{
  u32 *end;
  int rem;

  rem = nb&3;
  end = dest+(nb-rem);
  if(dest<end)
    do
    {
      *dest++ = sign;
      *dest++ = sign;
      *dest++ = sign;
      *dest++ = sign;
    } while(dest<end);
    if(rem&1)
      *dest++ = sign;
    if(rem&2)
    {
      *dest++ = sign;
      *dest++ = sign;
    }
}
