/**
 * @ingroup   sc68app_devel
 * @file      SC68play.c
 * @author    Ben(jamin) Gerard <ben@sashipa.com>
 * @date      1999/05/11
 * @brief     sc68 - play bar interface
 * @version   $Id: SC68play.c,v 1.1.1.1 2001/12/24 20:50:11 ben Exp $
 */

/*
 *                        sc68 - play bar interface
 *         Copyright (C) 2001 Ben(jamin) Gerard <ben@sashipa.com>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include "sc68app/SC68play.h"
#include "file68/SC68error.h"
#include "file68/SC68utils.h"

#include "file68/SC68os_general.h"
#include "sc68app/SC68os_interface.h"

static int check_disk(SC68app_t *sc68)
{
  if(sc68->cur_disk==NULL)
  {
    sc68->ready = 0;
  	return SC68error_add("No music file loaded");
  }
  return 0;
}

static int check_track(SC68app_t *sc68)
{
  /* Outside disk bound : set default */
  if(sc68->cur_track<0 || sc68->cur_track>=sc68->cur_disk->nb_six)
    sc68->cur_track = sc68->cur_disk->default_six;

  /* Default outside disk bound ! Disk is broken, should do an error */
  if(sc68->cur_track<0 || sc68->cur_track>=sc68->cur_disk->nb_six)
  {
    sc68->ready = 0;
    sc68->cur_mus = NULL;
    sc68->dis_mus = NULL;
    sc68->cur_track = -1;
    return SC68error_add("Internal error : Track outside Disk bound");
  }
  return 0;
}

/* Press PLAY ON TAPE
 *  -Desactiv PAUSE mode & activ PLAY mode
 *  -Load, Init & Start Track
 *  -Run callback
 *  -Return 0 if successed
*/
int SC68play(SC68app_t *sc68)
{
  int err;

/*  SC68os_pdebug("SC68play()\n");*/

  if(err=SC68app_check(sc68), err<0)
    return err;
  if(err=check_disk(sc68), err<0)
    return err;
  if(err=check_track(sc68), err<0)
    return err;

  if(sc68->ready && sc68->play && sc68->pause)
    return SC68pause(sc68);

  sc68->ready = 0;
  sc68->pause = 0;
  sc68->play  = 0;

  if(err=SC68app_start_current_track(sc68), err<0)
    return err;

  sc68->play  = 1;
  sc68->ready = 1;
  sc68->refresh_me = 1;
  return SC68os_interface_play_cb(sc68);
}

/* Press PAUSE ON TAPE
 *  -Toggle PAUSE mode
 *  -Run callback
 *  -Return 0 if successed
*/
int SC68pause(SC68app_t *sc68)
{
  int err;
  if(err=SC68app_check(sc68), err<0)
    return err;
  sc68->pause = !sc68->pause;
  return SC68os_interface_pause_cb(sc68);
}

/* Press STOP ON TAPE
 *  -Desactiv PLAY mode
 *  -Run callback if PLAY was activated
 *  -Return 0 if successed
*/
int SC68stop(SC68app_t *sc68)
{
  int err,modif;
  if(err=SC68app_check(sc68), err<0)
    return err;
  modif = sc68->play;
  sc68->play = 0;
  sc68->cur_mus = NULL;
  sc68->time.elapsed =
  sc68->time.sel_elapsed =
  sc68->time.rem = 0;
  if(modif)
    {
      sc68->refresh_me = 1;
      return SC68os_interface_stop_cb(sc68);
    }
  return 0;
}

/* Press FAST-FORWARD ON TAPE
 *  -Toggle FAST-FORWARD mode
 *  -Run callback
 *  -Return 0 if successed
*/
int SC68ffwd(SC68app_t *sc68)
{
  int err;
  if(err=SC68app_check(sc68), err<0)
    return err;
  sc68->ffwd = !sc68->ffwd;
  sc68->refresh_me = 1;
  return SC68os_interface_ffwd_cb(sc68);
}

/* Press TRACK-NUMBER ON TAPE
 *  -Select default track if number is outside bound
 *  -Run callback
 *  -Return 0 if successed
*/
int SC68track(SC68app_t *sc68, int track)
{
  int err;
  if(err=SC68app_check(sc68), err<0)
    return err;
  if(err=check_disk(sc68), err<0)
    return err;

  sc68->cur_track = track;
  if(err=check_track(sc68), err<0)
    return err;
  sc68->dis_mus = sc68->cur_disk->mus + sc68->cur_track;
  if(sc68->play)
  {
    if(err=SC68stop(sc68), err<0)
      return err;
    if(err=SC68play(sc68), err<0)
      return err;
  }
  sc68->refresh_me = 1;
  return SC68os_interface_track_cb(sc68);
}

/* Press NEXT-TRACK ON TAPE
 *  -Do nothing if already LAST TRACK
 *  -Run callback
 *  -Return 0 if successed
*/
int SC68ntrk(SC68app_t *sc68)
{
  int err;
  if(err=SC68app_check(sc68), err<0)
    return err;
  if(err=check_disk(sc68), err<0)
    return err;
  if(sc68->cur_track == sc68->cur_disk->nb_six-1)
    return 0;
  sc68->cur_track++;
  sc68->dis_mus++;
  if(sc68->play)
  {
/*    SC68os_pdebug("NTRK %u\n", sc68->cur_track);*/
    if(err=SC68stop(sc68), err<0)
      return err;
    if(err=SC68play(sc68), err<0)
      return err;
  } else {
/*    SC68os_pdebug("!! NOT PLAY NTRK %u\n", sc68->cur_track);*/
  }
  sc68->refresh_me = 1;
  return SC68os_interface_track_cb(sc68);
  return 0;
}

/* Press PREVIOUS-TRACK ON TAPE
 *  -Do nothing if already FIRST TRACK
 *  -Run callback
 *  -Return 0 if successed
*/
int SC68ptrk(SC68app_t *sc68)
{
  int err;
  if(err=SC68app_check(sc68), err<0)
    return err;
  if(err=check_disk(sc68), err<0)
    return err;
  if(sc68->cur_track == 0)
    return 0;
  sc68->cur_track--;
  sc68->dis_mus--;
  if(sc68->play)
  {
    if(err=SC68stop(sc68), err<0)
      return err;
    if(err=SC68play(sc68), err<0)
      return err;
  }
  sc68->refresh_me = 1;

  return SC68os_interface_track_cb(sc68);
}

/* Insert NEW DISK :
 *  -EJECT previous disk
 *  -LOAD new disk
 *  -PLAY
*/
int SC68load(SC68app_t *sc68, disk68_t *newdisk)
{
  int err;
  int play;
  if(err=SC68app_check(sc68), err<0)
    return err;
  play = sc68->play_at_load | sc68->play;
  if(err=SC68eject(sc68), err<0)
    return err;
  sc68->cur_disk = newdisk;
  if(err=SC68track(sc68,sc68->force_track-1), err<0)
  {
    sc68->cur_disk = NULL;
    return err;
  }
  if(play && (err=SC68play(sc68), err<0))
  {
    sc68->cur_disk = NULL;
    return err;
  }
  if(!SC68os_interface_load_cb(sc68))
    sc68->ready = 1;

  sc68->refresh_me = 1;

  return 0;
}

/* Press EJECT ON TAPE
 *  -STOP music
 *  -Run callback
 *  -Return 0 if successed
*/
int SC68eject(SC68app_t *sc68)
{
  int err;
  if(err=SC68stop(sc68), err<0)
    return err;
  sc68->ready = 0;
  sc68->cur_track = -1;
  sc68->cur_mus = NULL;
  sc68->dis_mus = NULL;

  sc68->time.track =
  sc68->time.selection = 0;

  sc68->refresh_me = 1;
  if(sc68->cur_disk!=NULL)
  {
    SC68_free(sc68->cur_disk);
    sc68->cur_disk = NULL;
    return SC68os_interface_eject_cb(sc68);
  }
  return 0;
}
